/*  www.sgm.siedlce.pl,  projekt do wykorzystania wylacznie na wlasny uzytek
    
  Program do walki powietrznej na podczerwien z wykorzystaniem
  modulu YS-IRTM, podlaczonego do we/wy RxD/TxD Arduino
  
  Funkcje:
  - detekcja sygnalu RC, dioda LED D13 miga kiedy jest sygnal RC,
    gasnie kiedy brak sygnalu, swieci ciaglym przez zadany czas
    po detekcji "trafienia", w tym czasie kolejne "trafienia" sa
    blokowane
  - ustawienie roznych kombinacji komend "strzal/trafienie",
    2 moduly musza zostac zaprogramowane z roznymi kombinacjami
  - komendy "strzalu" generowane automatycznie 3 razy na sek.
  - zalozona zworka blokuje generowanie komend "strzalu" 
    
  - pomiar czasu przy uzyciu biblioteki (autor Gabriela Staples, licencja GNU)
===================================================================================================
<eRCaGuy_Timer2_Counter.h> by Gabriel Staples
Visit my blog at http://electricrcaircraftguy.blogspot.com/
-My contact info is available by clicking the "Contact Me" tab at the top of my blog.
-Please support my work & contributions by buying something here: https://sites.google.com/site/ercaguystore1/
My original post containing this code can be found here: http://electricrcaircraftguy.blogspot.com/2014/02/Timer2Counter-more-precise-Arduino-micros-function.html
Written: 17 May 2014
Updated: 30 May 2014
===================================================================================================
  LICENSE & DISCLAIMER
  Copyright (C) 2015 Gabriel Staples.  All right reserved.
  
  ------------------------------------------------------------------------------------------------
  License: GNU Lesser General Public License Version 3 (LGPLv3) or later - https://www.gnu.org/licenses/lgpl.html
  ------------------------------------------------------------------------------------------------

  This file is part of eRCaGuy_Timer2_Counter.
  
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program.  If not, see http://www.gnu.org/licenses/
===================================================================================================

*/

//dolaczenie biblioteki
#include <eRCaGuy_Timer2_Counter.h>

// funkcja timer2.get_count()  podaje wynik w jednostkach 0.5us
// funkcja timer2.get_micros() podaje wynik w mikrosekundach




//deklaracje stalych zmiennych - parametry do edycji

const byte cmdSet = 0;      // 0 lub 1 w zaleznosci od wersji zestawu komend IR
                            // jeden modul musi byc zaprogramowany z cmdSet = 0
                            // drugi modul musi byc zaprogramowany z cmdSet = 1
                            
const int tGapSh = 2000;    // po trafieniu - zwloka z gaz na min i blokada odbiorania IR
                            // czas w msec

const int tGapSD = 1000;    // po trafieniu - blokada odbierania IR po odblokowaniu gazu
                            // czas w msec
                            
int tGapF = 300;            // co ile (msec) generowana automatycznie komenda strzalu


//pozostale deklaracje stalych zmiennych

//obsluga detekcji PPM
volatile unsigned long pulse_start = 0; //zmienna dla get_count()  
volatile unsigned long pulse_uS = 0;    //zmienna na dlugosc impulsu kanalowego                              
int pulse_uSi;                          //zmienna pomocnicza 
byte pulseOK = 0;                       //jest prawidlowy impuls PPM

//obsluga serwa
#include <Servo.h>               // dołączenie biblioteki 'servo'
Servo myservo;                   // utworzenie obiektu do sterowania serwem 
const int servoFS = 1100;        //warosc FailSafe
int servoP;                      //zmienna na sygnal wyjsciowy servo

//obsluga wydruku (opcja)
int tGap = 300;                  //co ile wydruk(msec) - opcjonalnie
unsigned long t_start;           //zmienne do odliczania czasu przy wydruku
unsigned long t_count; 

//obsluga strzalu ir (tryb automatyczny)
//int tGapF = 300;                //co ile strzal(msec) automatyczny (przeniesione do gory)
unsigned long t_startF;           //zmienne do odliczania petli strzalu
unsigned long t_countF; 

//obsluga LED
const int LED = 13;        //LED na Pin 13
byte LED_status = 0;       //status diody LED
const int tGapLED = 330;   //obsluga migania LED (sygnal RC)
unsigned long t_startL;    //zmienne do pomiaru czasu
unsigned long t_countL;


//obsluga modulu IR
const byte prefix1 =0xA1;  //0xA1 obowiazkowy pierwszy byte YS-IRTM (nadawanie)  
const byte prefix2 =0xF1;  //0xF1 obowiazkowy drugi byte YS-IRTM (nadawanie)
const byte cmd1 = 0x00;    // 1. byte komendy IR
const byte cmd2 = 0xFF;    // 2. byte komendy IR
byte cmd3R;                // 3. byte komendy IR, zmienna, odbior, wart. zaleznie od zworki
byte cmd3T;                // 3. byte komendy IR, zmienna, nadawanie, wart. zaleznie od zworki
const byte cmdA = 0x45;    // komenda 0x45 (Ch-)
const byte cmdB = 0x47;    // komenda 0x47 (Ch=)
byte cmdVer;               // zmienna do wyboru wersji modulu (czym strzelam, co mnie trafia)
byte bufCmd[2];            // 3-bajtowy bufor komend
byte shotRC = 0;           // obsluga trafienia
byte shotStatus = 0;  

//obsluga zaniku sygnalu RC z generowaniem FS dla przepustnicy
const int tGapPC = 250;   //okres testowania sygnalu RC
unsigned long t_startP;
unsigned long t_countP;
unsigned int P_Count = 0;
byte Ptest_status = 0; 

int ZW1;                   //zmienna do odczytu wejscia A0


//inicjalizacja programu
void setup() {

  // inicjalizacja serwa do Pin12 
  myservo.attach(12);      // serwo na D12
  myservo.writeMicroseconds(servoFS);
  servoP = servoFS;
  
  //inicjalizacja LED
  pinMode(LED, OUTPUT);
  digitalWrite(LED, HIGH); 
  
  //obowiazkowa inicjalizacja Timer2 wymagana przez
  //biblioteke <eRCaGuy_Timer2_Counter.h>
  timer2.setup(); 
  
  //inicjalizacja portu szeregowego (opcjonalnie, prezentacja danych)
  Serial.begin(9600); 
  
  //inicjalizacja przerwania 0 (pin 2, Arduino UNO)
  attachInterrupt(0, pulse_rising, RISING);
  t_start = millis();
  
  //przypisanie zestawu komend Rx/Tx
  cmdVer = cmdSet;     // 0 lub 1 w zaleznosci od wersji modulu     
  //obsluga wersji moduluc 
  if (cmdVer==0) {
    cmd3R = cmdA;         
    cmd3T = cmdB;
  } else {
    cmd3R = cmdB;         
    cmd3T = cmdA;    
  } //if (cmdVer==0)
 
  delay(300);
  digitalWrite(LED, LOW); 
  
}

//glowna petla programu
void loop() {
  // tutaj program uzytkowy, bazujacy na zmierzonym
  // czasie trwania impulsu kanlaowego PPM
  
  byte inByte;
  while (Serial.available() > 0)
  {
    inByte = Serial.read();
  }
 
     if (inByte == cmd3R) {
     //jest trafienie
     inByte = 0;  
     shotRC = shotRC + 1;
     }
   
  if ((shotStatus == 0) && (shotRC > 0)){  
   shotStatus = 1;
  }  
  
  
  
  //opcjonalnie obsulga drukowania co 'tGap' pomiary na port szeregowy  
  /*
  t_count = millis(); 
  //jesli uplynelo tGap to drukuj
  if ((t_count - t_start) >= tGap) 
  {
    t_start = t_count;
    //tu drukuje testowane parametry
    //Serial.println(servoP);
  }
  */
 
  
   //obsluga testu sygnalu RC
   t_countP = millis();
   if ((t_countP - t_startP) >= tGapPC) {
    t_startP = t_countP; 
    if (Ptest_status == 0) {
      //digitalWrite(LED, HIGH);
      Ptest_status = 1;
      if (P_Count > 0){ //brak sygnalu RC
       pulseOK = 1;
      } else {
       pulseOK = 0; 
      }
     
   } else {
       //digitalWrite(LED, LOW);
       Ptest_status = 0;
       P_Count = 0; 
     } 
    
   }
 
 if (pulseOK == 1) {  //miganie jesli PPM OK 
 

   t_countF = millis(); 
   //jesli uplynelo tGapF to strzal
   if ((t_countF - t_startF) >= tGapF) 
   {
     t_startF = t_countF;
     ir_fire(); 
   }

 
  //obsluga migania diody LED
  switch (shotStatus) {
    
  case 0:    //nie ma trafienia, miganie normalne
   myservo.writeMicroseconds(servoP);
   t_countL = millis();
   if ((t_countL - t_startL) >= tGapLED) {
   // if (pulseOK == 1) {  //miganie jesli PPM OK
     t_startL = t_countL; 
     if (LED_status == 0) {
      digitalWrite(LED, HIGH);
      LED_status = 1; 
     } else {
       digitalWrite(LED, LOW);
       LED_status = 0;
       //pulseOK = 0;  //zmiana testu
     } 
    //}  //if (pulseOK == 1)
    
   } //if ((t_countL - t_startL) >= tGapLED)
  
   break; // case 0
   
   case 1: //jest trafienie, dioda swieci 1sek
    
    digitalWrite(LED, HIGH);
    LED_status = 1;
    myservo.writeMicroseconds(servoFS);

    t_countL = millis();
    if ((t_countL - t_startL) >= tGapSh) {
     t_startL = t_countL; 
     //digitalWrite(LED, LOW);
     //LED_status = 0;
     shotStatus = 2;
     
    } 
   break;  // case 1
   
   case 2: //zwloka po trafieniu
    myservo.writeMicroseconds(servoP);
    t_countL = millis();
    digitalWrite(LED, HIGH);
    LED_status = 1;
    if ((t_countL - t_startL) >= tGapSD) {
     t_startL = t_countL;
     digitalWrite(LED, LOW);
     LED_status = 0; 
     shotStatus = 0;
     shotRC=0; 
    }    

   break;
   
  } //switch (shotStatus)  
  
 } else { //brak impulsu RC
 
  digitalWrite(LED, LOW);
  myservo.writeMicroseconds(servoFS);

 } //if (pulseOK == 1)
 
} // koniec glownej petli


// generowanie strzalu
void ir_fire() {  //tmp wylaczone
  //if (digitalRead(PinJP)==1) { //jesli brak zworki strzelaj
  ZW1 = analogRead(0);
  delay(5);
  if (ZW1 > 300) {         //strzelaj jesli zworka zdjeta (+5V)
    Serial.write(prefix1);
    Serial.write(prefix2);
 //    //Serial.write(cmd1);
 //    //Serial.write(cmd2);
    Serial.write(cmd3T);
  } 
}


// obsluga przerwania (rising), poczatek impulsu
void pulse_rising() {
  pulse_start = timer2.get_count();
  attachInterrupt(0, pulse_falling, FALLING);  
}

// obsluga przerwania (falling) 
void pulse_falling() {
  //dlugosc impulsu w microsec()
  pulse_uS = (timer2.get_count()-pulse_start)/2;
  pulse_uSi = pulse_uS;
  //sprawdzenie czy dlugosc impulsu z dozwolonego zakresu
  if ((pulse_uSi > 900) && (pulse_uSi < 2100)) {
     servoP = pulse_uSi;
     //pulseOK = 1;      //zmiana testu detekcji RC
     P_Count = P_Count + 1;
     //myservo.writeMicroseconds(servoP);
  }   //if ((pulse1_getm > 900
  
  
  
  attachInterrupt(0, pulse_rising, RISING);
  timer2.reset();  // ważne, resetowanie timera!
           
}





